Skip to contents

Assume that we consider an ergodic/irreducible time-homogeneous Markov chain specified using a policy in the MDP.

Usage

get_steady_state_pr(mdp, get_log = FALSE)

Arguments

mdp

The MDP loaded using load_mdp().

get_log

Output log text.

Value

A vector with steady state probabilities for all the states at the founder level.