Skip to contents

Return the index of a weight in the model. Note that index always start from zero (C++ style), i.e. the first weight, the first state at a stage etc has index 0.

Usage

get_w_idx(mdp, w_lbl)

Arguments

mdp

The MDP loaded using load_mdp().

w_lbl

The label/string of the weight.

Value

The index (integer).