Calculate the steady state transition probabilities for the founder process (level 0).
Source:R/mdp.R
get_steady_state_pr.RdAssume that we consider an ergodic/irreducible time-homogeneous Markov chain specified using a policy in the MDP.
Arguments
- mdp
The MDP loaded using
load_mdp().- get_log
Output log text.